/**
 * This interface is needed to bridge between different segment implementations
 * with Soya's ReduxStore implementation.
 *
 * Segment represents a segment of our unified store tree, specifically, a top
 * level key in our state tree:
 *
 * <pre>
 *   state = {
 *     segmentA: {...},
 *     segmentB: [...],
 *     ...
 *   };
 * </pre>
 *
 * IMPORTANT NOTE: All methods described here is considered non public and
 * should be overridden by child class.
 *
 * @CLIENT_SERVER
 */'use strict';Object.defineProperty(exports, '__esModule', { value: true });var _createClass = (function () {function defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ('value' in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}return function (Constructor, protoProps, staticProps) {if (protoProps) defineProperties(Constructor.prototype, protoProps);if (staticProps) defineProperties(Constructor, staticProps);return Constructor;};})();function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError('Cannot call a class as a function');}}var 
Segment = (function () {function Segment() {_classCallCheck(this, Segment);}_createClass(Segment, [{ key: '_activate', 
    /**
     * Called by ReduxStore when this Segment is registered.
     *
     * IMPORTANT NOTE: This operation should be idempotent.
     *
     * @param {Function} PromiseImpl
     */value: 
    function _activate(PromiseImpl) {}



    /**
     * Called by ReduxStore when no component subscribes to this Segment anymore.
     * This is used to stop polling or other background processes that might
     * be running.
     *
     * IMPORTANT NOTE: This operation should be idempotent;
     */ }, { key: '_deactivate', value: 
    function _deactivate() {}



    /**
     * Called by ReduxStore when no component subscribes to the given Segment
     * query anymore. This is used to stop polling or other background processes
     * related to the given query.
     *
     * IMPORTANT NOTE: This operation should be idempotent.
     *
     * @param {string} queryId
     */ }, { key: '_deactivateQuery', value: 
    function _deactivateQuery(queryId) {}



    /**
     * Returns query ID. To be used by ReduxStore in identifying queries.
     *
     * @param {any} query
     * @param {Object} options
     * @return {string}
     */ }, { key: '_registerQuery', value: 
    function _registerQuery(query, options) {}



    /**
     * Uses action creator to create load action of the given query.
     *
     * @param {string} queryId
     * @return {Object | Function}
     */ }, { key: '_createHydrateAction', value: 
    function _createHydrateAction(queryId) {}



    /**
     * Get the segment name. Segment's name is hard-coded by each segment
     * implementation and must never change.
     *
     * @return {string}
     */ }, { key: '_getName', value: 
    function _getName() {
      return '';}


    /**
     * When called, returns the ActionCreator instance. Unlike reducer,
     * ActionCreator can be a stateful object. This is allowed since ActionCreator
     * has to deal with caching and AJAX requests.
     *
     * @return {ActionCreator}
     */ }, { key: '_getActionCreator', value: 
    function _getActionCreator() {}



    /**
     * Returns an object containing data and errors.
     *
     * @param {any} state
     * @param {string} queryId
     * @return {{data: ?any; errors: ?any}}
     */ }, { key: '_getPieceObject', value: 
    function _getPieceObject(state, queryId) {}



    /**
     * Compares two segment states, returns true if the segment state is
     * different.
     *
     * @param {any} segmentStateA
     * @param {any} segmentStateB
     * @return {boolean}
     */ }, { key: '_isStateEqual', value: 
    function _isStateEqual(segmentStateA, segmentStateB) {}



    /**
     * Compares pieces of two state. If they are equal return null, otherwise
     * return piece of the current segment state.
     *
     * @param prevSegmentState
     * @param segmentState
     * @param queryId
     * @return {?{data: ?any; errors: ?any}}
     */ }, { key: '_comparePiece', value: 
    function _comparePiece(prevSegmentState, segmentState, queryId) {}



    /**
     * Returns a reducer function. Called only once by Store, on registration
     * of a new segment.
     *
     * Reducer is responsible for:
     * 1) Initializing default segment state.
     * 2) Handling changes to segment state.
     *
     * NOTE: We could easily convert this into reduce() method. However in the
     * spirit of redux, reducers should be stateless functions. Let state of
     * subscription, queries, etc. be handled by others. We're making it
     * much simpler this way, methinks.
     *
     * @return {Function}
     */ }, { key: '_getReducer', value: 
    function _getReducer() {} }]);return Segment;})();exports['default'] = Segment;module.exports = exports['default'];
//# sourceMappingURL=Segment.js.map