/**
 * Default logging class. Logging is by default asynchronous, but no callbacks
 * will be called (why should you?). Please note that this class is only used
 * by the framework to log on server. It does not dictate how other library
 * might log their info/error messages.
 *
 * Logging levels shamelessly copied from Seldaek's awesome PHP Monolog library.
 *
 * IMPORTANT NOTE: It is assumed that calls to log will not be treated as a
 * potentially dangerous call (i.e. calls will not be wrapped in try catch).
 * You should think about your own use case, that is, whether or not to throw
 * exception is logging fails.
 *
 * @SERVER
 */'use strict';Object.defineProperty(exports, '__esModule', { value: true });var _createClass = (function () {function defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ('value' in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}return function (Constructor, protoProps, staticProps) {if (protoProps) defineProperties(Constructor.prototype, protoProps);if (staticProps) defineProperties(Constructor, staticProps);return Constructor;};})();function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError('Cannot call a class as a function');}}var 
Logger = (function () {


  /**
   * TODO: Make it so that by setting error logging settings, we can determine which messages to actually log.
   * @param {boolean} isProduction
   */
  function Logger(isProduction) {_classCallCheck(this, Logger);
    this.shouldDebug = !isProduction;}


  /**
   * Prep node's incoming message object for logging. You can override this
   * method to log just the right amount of information.
   *
   * @param {http.incomingMessage} httpRequest
   * @return {Array<string>}
   */_createClass(Logger, [{ key: 'prepRequest', value: 
    function prepRequest(httpRequest) {
      return [
      httpRequest.method, 
      httpRequest.url, 
      httpRequest.headers];}



    /**
     * Prep node's http server response object for logging. You can override this
     * method to log just the right amount of information.
     *
     * @param {httpServerResponse} response
     */ }, { key: 'prepResponse', value: 
    function prepResponse(response) {
      // TODO: Modify the framework so we can at least show the body?
      // Node unfortunately doesn't allow us to show what is already sent to
      // the user, so there isn't much information we can log here.
      return [
      response.statusCode, 
      response.statusMessage];}



    /**
     * Detailed debug information. Ignored in production.
     */ }, { key: 'debug', value: 
    function debug(message, error, extras) {
      if (shouldDebug) this._print('[DEBUG] ' + message, error, extras);}


    /**
     * Normal events. Example: user logs in, SQL logs.
     */ }, { key: 'info', value: 
    function info(message, error, extras) {
      this._print('[INFO] ' + message, error, extras);}


    /**
     * Normal but interesting events.
     */ }, { key: 'notice', value: 
    function notice(message, error, extras) {
      this._print('[NOTICE] ' + message, error, extras);}


    /**
     * Exceptional occurrences that are not errors. Example: usage of deprecated
     * APIs, poor use of API, undesirable things that are not errors (yet).
     */ }, { key: 'warning', value: 
    function warning(message, error, extras) {
      this._print('[WARNING] ' + message, error, extras);}


    /**
     * Runtime errors that do not require immediate action but nevertheless should
     * be logged and monitored.
     */ }, { key: 'error', value: 
    function error(message, _error, extras) {
      this._print('[ERROR] ' + message, _error, extras);}


    /**
     * Action must be taken immediately. Example: entire website down, state in
     * server is undefined so that it needs to be restarted, database unavailable,
     * ideally you should override this method to send yourself an SMS to wake
     * you up.
     */ }, { key: 'alert', value: 
    function alert(message, error, extras) {
      this._print('[ALERT] ' + message, error, extras);}


    /**
     * Your pantsu is on fire.
     */ }, { key: 'emergency', value: 
    function emergency(message, error, extras) {
      this._print('[EMERGENCY] ' + message, error, extras);} }, { key: '_print', value: 


    function _print(message, error, extras) {
      console.log(message, error ? error.stack ? error.stack : error : '', extras ? extras : '');} }]);return Logger;})();exports['default'] = Logger;module.exports = exports['default'];
//# sourceMappingURL=Logger.js.map